"""
Created on 8 mars 2018

@author: Tonneau Stephane
"""
import datetime
from pidclient.base_utils import JsonBased
from pidclient.base_utils import GeoJSON

class Metainformation(JsonBased):
    """Class that encapsulate some metainformation of pid entries"""

    def __init__(self, uid=None):
        """
        Initialisator of the class

        :param uid: The unique ID of the object
        :type uid: string
        """

        self.uid = uid
        "The unique ID of the object"
        self.timestamp = None
        "The creation timestamp "

class FileInfo(JsonBased):
    """Class that encapsulate a file information"""

    INPUT_CONTEXT = "input"
    CREATED_CONTEXT = "created"
    REMOVED_CONTEXT = "removed"

    def __init__(self, path, context=CREATED_CONTEXT, type_value=None, geometry=None,
                 checksum=None, size=None, product_id=None, ogc_urn=None):
        """
        Initialisator of the class

        :param path: absolute path to the file or directory
        :param context: the context associated with the file
        :param type_value: the media type associated with the file
        :param geometry: the geometry in the GeoJSON format
        :param checksum: the checksum of the file in the format {[SHA|MD5|..]}<checksum value>
        :param size: the total size of the file or directory
        :param product_id: the product_id of the product to which the file or directory refers
        :param ogc_urn: the ogc urn reference of the product to which the file or directory refers
        :type path: string
        :type context: string
        :type type_value: string
        :type geometry: wkt geometry
        :type checksum: int
        :type size: string
        :type product_id: string
        :type ogc_urn: string
        """

        self.context = context
        "the context associated with the file"
        self.type = type_value
        "the media type associated with the file"
        self.path = path
        "absolute path to the file or directory"
        if geometry is not None:
            self.geometry = GeoJSON(geometry)
        self.checksum = checksum
        "the checksum of the file in the format {[SHA|MD5|..]}<checksum value>"
        self.size = size
        "the total size of the file or directory"
        self.product_id = product_id
        "the product_id of the product to which the file or directory refers"
        self.ogc_urn = ogc_urn
        "The ogc urn reference of the product to which the file or directory refers"

class TraceInfo(JsonBased):
    """Class that encapsulate the execction trace"""

    def __init__(self, trace):
        """
        Initialisator of the class

        :param trace: an execution trace (commands with parameters, ...)
        :type trace: string
        """

        self.trace = trace
        "the information associated with the Trace"

class PidEntry(JsonBased):
    """Class that can be used to store the pertinent process log information"""

    def __init__(self, product_id, product_type, product_date, job_id=None,
                 proc_type=None, environment=None, start_time=None, user=None, host=None,
                 job_desc=None, job_type=None, parent_job_id=None, end_time=None,
                 exit_code=None, message=None):
        """
        Initialisator of the class

        :param product_id: id of the product that the process has manipulated
        :param product_type: Type of the product that the process has manipulated
        :param product_date: creation date of the product that the process has manipulated
        :param job_id: id of the process
        :param proc_type: type of job execute by the process
               Can be initialize, download, pretest, configure, process, posttest,
               quicklook, validate, archive, clean,...
        :param environment: environment on which the process was executed
        :param start_time: start time of the execution of the process
        :param user: user under which the process was executed
        :param host: hostname on which the process was executed"
        :param job_desc: Informal description of the process
        :param job_type: type of process executed by the process
               Can be a Yarn/Spark application ID (application_1234567),
               or a workflow job id (Airflow/Oozie/...)
        :param parent_job_id: id of the parent of the process
        :param end_time: end time of the execution of the process
        :param exit_code: exit code of the process
        :param message: message associated to the process
        :type product_id: string
        :type product_type: string
        :type product_date: date
        :type job_id: string
        :type proc_type: string
        :type environment: string
        :type start_time: date
        :type user: string
        :type host: string
        :type job_desc: string
        :type job_type: string
        :type parent_job_id: string
        :type end_time: date
        :type exit_code: int
        :type message: string
        """

        self.product_id = product_id
        """id of the created product"""
        self.product_type = product_type
        """Type of the product of the created product"""
        self.product_date = product_date
        """creation date of the created product"""
        self.job_desc = job_desc
        """Informal description of the process"""
        self.proc_type = proc_type
        """type of job execute by the process"""
        self.parent_job_id = parent_job_id
        """id of the parent of the process"""
        self.job_id = job_id
        """id of the process"""
        self.job_type = job_type
        """type of process executed by the process"""
        self.start_time = start_time
        """start time of the execution of the process"""
        self.end_time = end_time
        """end time of the execution of the process"""
        self.duration = None
        """duration of the process"""
        self.exit_code = exit_code
        """exit code of the process"""
        self.message = message
        """message associated to the process"""
        self.user = user
        """user under which the process was executed"""
        self.host = host
        """hostname on which the process was executed"""
        self.environment = environment
        """environment on which the process was executed"""
        self.files = []
        """FileInfo objects describing files manipulated/created/... by the process"""
        self.execution_info = []
        """TraceInfo objects describing execution traces of the process"""
        self.configuration = None
        """Sparc configuration Object associated to the process"""
        self.id = None
        """Unique id of this entry (populated if required)"""

    def finalise(self):
        """
        update the current process log information by
        fixing the endtime and the duration.

        :returns: the object itself
        :rtype: PidEntry
        """
        self.end_time = datetime.datetime.now()
        self.duration = (
            self.end_time - self.start_time).total_seconds()
        return self
