"""
Created on 26 mars 2018

@author: Stephane Tonneau

This module requires the installation of the elasticsearch library with
    pip install elasticsearch

"""

try:
    from elasticsearch import Elasticsearch
except ImportError:
    try:
        from elasticsearch import Elasticsearch
    except ImportError:
        try:
            from .elasticsearch import Elasticsearch
        except ImportError:
            print("ERROR : missing Elasticsearch package")

from pidclient.base_utils import LoggingMethod, Tracer

NODENAME = "elasticsearch"
# ESINDEX = "testprobav4"
# ESTYPE = "products4"


def _get_elasticsearch(urls):
    """return the elastic elastic search parameters

    :param urls: a string representation of a list of urls separated by commas
    :type urls: string
    :returns: an initialisated Elasticsearch object if urls are provided, or None otherwise
    :rtype: Elasticsearch

    """
    if urls is None:
        return None
    return Elasticsearch(
        urls.split(","),
        # sniff before doing anything
        sniff_on_start=False,
        # refresh nodes after a node fails to respond
        sniff_on_connection_fail=False,
        # and also every 60 seconds
        sniffer_timeout=60
    )


class ElasticsearchLogging(LoggingMethod):
    """Class that encapsulates the storing of objects in elasticsearch
    for logging purposes."""

    def __init__(self, **parameters):
        """Initialisator of the class

        :param configuration: the general logging configuration
        :type configuration: a python configuration object
        """
        super(ElasticsearchLogging, self).__init__(**parameters)
        es_urls = self.get_config(NODENAME, "hosts")
        if es_urls is None:
            Tracer.error("ElasticsearchLogging",
                         "Missing urls configuration value")
        self.esindex = self.get_config(NODENAME, "index")
        if self.esindex is None:
            Tracer.error("ElasticsearchLogging",
                         "Missing index configuration value")
        self.estype = self.get_config(NODENAME, "type")
        if self.estype is None:
            Tracer.error("ElasticsearchLogging",
                         "Missing type configuration value")
        try:
            self.es_connector = _get_elasticsearch(es_urls)
        except:
            self.es_connector = None
        if self.es_connector is None:
            Tracer.error("ElasticsearchLogging",
                         "module not configured properly")

    def close(self):
        """Close the logging instance"""
        pass

    def process_log(self, entry, meta_information):
        """Method to index a dictionary representation of both an object
        and its metadata in elastic search
        system

        :param entry: the standard information to log
        :param meta_information: meta information about the information to log
        :type entry: PidEntry
        :type meta_information: Metainformation
        """
        res = None
        if self.es_connector is not None:
            try:
                doc = entry.to_dict()
                res = self.es_connector.index(
                    index=self.esindex, doc_type=self.estype, id=meta_information.uid, body=doc)
            except:
                Tracer.error("ElasticsearchLogging",
                             "Unable to write the logging entry")
        return res
